"""
points_from_coordinates
=======================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "mesh" category
"""

class points_from_coordinates(Operator):
    """Extract a mesh made of points elements. This mesh is made from input meshes coordinates on the input scopings.

      available inputs:
        - nodes_to_keep (Scoping, ScopingsContainer)
        - mesh (MeshedRegion, MeshesContainer)

      available outputs:
        - meshed_region (MeshedRegion)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mesh.points_from_coordinates()

      >>> # Make input connections
      >>> my_nodes_to_keep = dpf.Scoping()
      >>> op.inputs.nodes_to_keep.connect(my_nodes_to_keep)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mesh.points_from_coordinates(nodes_to_keep=my_nodes_to_keep,mesh=my_mesh)

      >>> # Get output data
      >>> result_meshed_region = op.outputs.meshed_region()"""
    def __init__(self, nodes_to_keep=None, mesh=None, config=None, server=None):
        super().__init__(name="mesh::points_from_coordinates", config = config, server = server)
        self._inputs = InputsPointsFromCoordinates(self)
        self._outputs = OutputsPointsFromCoordinates(self)
        if nodes_to_keep !=None:
            self.inputs.nodes_to_keep.connect(nodes_to_keep)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Extract a mesh made of points elements. This mesh is made from input meshes coordinates on the input scopings.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "nodes_to_keep", type_names=["scoping","scopings_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "meshed_region", type_names=["abstract_meshed_region"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mesh::points_from_coordinates")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsPointsFromCoordinates 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsPointsFromCoordinates 
        """
        return super().outputs


#internal name: mesh::points_from_coordinates
#scripting name: points_from_coordinates
class InputsPointsFromCoordinates(_Inputs):
    """Intermediate class used to connect user inputs to points_from_coordinates operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.points_from_coordinates()
      >>> my_nodes_to_keep = dpf.Scoping()
      >>> op.inputs.nodes_to_keep.connect(my_nodes_to_keep)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(points_from_coordinates._spec().inputs, op)
        self._nodes_to_keep = Input(points_from_coordinates._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._nodes_to_keep)
        self._mesh = Input(points_from_coordinates._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def nodes_to_keep(self):
        """Allows to connect nodes_to_keep input to the operator

        Parameters
        ----------
        my_nodes_to_keep : Scoping, ScopingsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.points_from_coordinates()
        >>> op.inputs.nodes_to_keep.connect(my_nodes_to_keep)
        >>> #or
        >>> op.inputs.nodes_to_keep(my_nodes_to_keep)

        """
        return self._nodes_to_keep

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.points_from_coordinates()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsPointsFromCoordinates(_Outputs):
    """Intermediate class used to get outputs from points_from_coordinates operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.points_from_coordinates()
      >>> # Connect inputs : op.inputs. ...
      >>> result_meshed_region = op.outputs.meshed_region()
    """
    def __init__(self, op: Operator):
        super().__init__(points_from_coordinates._spec().outputs, op)
        self._meshed_region = Output(points_from_coordinates._spec().output_pin(0), 0, op) 
        self._outputs.append(self._meshed_region)

    @property
    def meshed_region(self):
        """Allows to get meshed_region output of the operator


        Returns
        ----------
        my_meshed_region : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.points_from_coordinates()
        >>> # Connect inputs : op.inputs. ...
        >>> result_meshed_region = op.outputs.meshed_region() 
        """
        return self._meshed_region




"""
Check the matching for a client/server pair.

Used to verify if the server version is a minimum value.
"""

from ansys.dpf.core import errors as dpf_errors
import sys


def server_meet_version(required_version, server):
    """Check if a given server version matches with a required version.

    Parameters
    ----------
    required_version : str
        Required version to compare with the server version.
    server : :class:`ansys.dpf.core.server`
        DPF server object.

    Returns
    -------
    bool
        ``True`` when successful, ``False`` when failed.
    """
    version = get_server_version(server)
    meets = version_tuple(required_version)
    return meets_version(version, meets)


def server_meet_version_and_raise(required_version, server, msg=None):
    """Check if a given server version matches with a required version and raise
    an execepton if it doesn not match.

    Parameters
    ----------
    required_version : str
        Required version to compare with the server version.
    server : :class:`ansys.dpf.core.server`
        DPF server object.
    msg : str, optional
        Message contained in the raised exception if the versions do
        not match. The default is ``None``, in which case the default message
        is used.

    Raises
    ------
    dpf_errors : errors
        errors.DpfVersionNotSupported is raised if the versions do not match.

    Returns
    -------
    bool
        ``True`` when successful, ``False`` when failed.
    """

    if not server_meet_version(required_version, server):
        if msg is not None:
            raise dpf_errors.DpfVersionNotSupported(required_version, msg=msg)
        else:
            raise dpf_errors.DpfVersionNotSupported(required_version)
    # server meets version if ends here
    return True


def meets_version(version, meets):
    """Check if a version string meets a minimum version.

    Parameters
    ----------
    version : str
        Version string to check. For example, ``"1.32.1"``.
    meets : str
        Required version for comparison. For example, ``"1.32.2"``.

    Returns
    -------
    bool
         ``True`` when successful, ``False`` when failed.
    """
    if not isinstance(version, tuple):
        va = version_tuple(version)
    else:
        va = version

    if not isinstance(meets, tuple):
        vb = version_tuple(meets)
    else:
        vb = meets

    if len(va) != len(vb):
        raise ValueError("Versions are not comparable.")

    for i in range(len(va)):
        if va[i] > vb[i]:
            return True
        elif va[i] < vb[i]:
            return False

    return True


def get_server_version(server=None):
    """Retrieve the server version as a string.

    Parameters
    ----------
    server : :class:`ansys.dpf.core.server`, optional
        DPF server object. The default is ``None``.

    Returns
    -------
    str
        Server version.
    """
    from ansys.dpf import core

    if server is None:
        version = core.SERVER.version
    else:
        version = server.version
    return version


def version_tuple(ver):
    """Convert a version string to a tuple containing integers.

    Parameters
    ----------
    ver : str
        Version string to convert.

    Returns
    -------
    ver_tuple : tuple
        Three-part tuple representing the major, minor, and patch
        versions.
    """
    split_ver = ver.split(".")
    while len(split_ver) < 3:
        split_ver.append("0")

    if len(split_ver) > 3:
        raise ValueError(
            "Version strings containing more than three parts " "cannot be parsed"
        )

    vals = []
    for item in split_ver:
        if item.isnumeric():
            vals.append(int(item))
        else:
            vals.append(0)

    return tuple(vals)


def version_requires(min_version):
    """Check that the method being called matches a certain server version.

    .. note::
       The method must be used as a decorator.
    """

    def decorator(func):
        # first arg *must* be a tuple containing the version
        if not isinstance(min_version, str):
            raise TypeError(
                "version_requires decorator must be a string with a dot separator."
            )

        def wrapper(self, *args, **kwargs):
            """Call the original function"""
            server = self._server
            func_name = func.__name__
            class_name = self.__class__.__name__

            # particular cases
            # scoping._set_ids case, must be checked in a particular way
            if func_name == "_set_ids" and class_name == "Scoping":
                ids = args[0]
                size = len(ids)
                if size != 0:
                    max_size = 8.0e6 // sys.getsizeof(ids[0])
                    if size > max_size:
                        server.check_version(min_version)
            # default case, just check the compatibility
            else:
                server.check_version(min_version)

            return func(self, *args, **kwargs)

        return wrapper

    return decorator



"""
add_fc
======
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "math" category
"""

class add_fc(Operator):
    """Select all fields having the same label space in the input fields container, and add those together. If fields, doubles, or vectors of doubles are put in input, they are added to all the fields.

      available inputs:
        - fields_container1 (FieldsContainer, Field, float, list)
        - fields_container2 (FieldsContainer, Field, float, list)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.add_fc()

      >>> # Make input connections
      >>> my_fields_container1 = dpf.FieldsContainer()
      >>> op.inputs.fields_container1.connect(my_fields_container1)
      >>> my_fields_container2 = dpf.FieldsContainer()
      >>> op.inputs.fields_container2.connect(my_fields_container2)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.add_fc(fields_container1=my_fields_container1,fields_container2=my_fields_container2)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container1=None, fields_container2=None, config=None, server=None):
        super().__init__(name="add_fc", config = config, server = server)
        self._inputs = InputsAddFc(self)
        self._outputs = OutputsAddFc(self)
        if fields_container1 !=None:
            self.inputs.fields_container1.connect(fields_container1)
        if fields_container2 !=None:
            self.inputs.fields_container2.connect(fields_container2)

    @staticmethod
    def _spec():
        spec = Specification(description="""Select all fields having the same label space in the input fields container, and add those together. If fields, doubles, or vectors of doubles are put in input, they are added to all the fields.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container","field","double","vector<double>"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "fields_container", type_names=["fields_container","field","double","vector<double>"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "add_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsAddFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsAddFc 
        """
        return super().outputs


#internal name: add_fc
#scripting name: add_fc
class InputsAddFc(_Inputs):
    """Intermediate class used to connect user inputs to add_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.add_fc()
      >>> my_fields_container1 = dpf.FieldsContainer()
      >>> op.inputs.fields_container1.connect(my_fields_container1)
      >>> my_fields_container2 = dpf.FieldsContainer()
      >>> op.inputs.fields_container2.connect(my_fields_container2)
    """
    def __init__(self, op: Operator):
        super().__init__(add_fc._spec().inputs, op)
        self._fields_container1 = Input(add_fc._spec().input_pin(0), 0, op, 0) 
        self._inputs.append(self._fields_container1)
        self._fields_container2 = Input(add_fc._spec().input_pin(1), 1, op, 1) 
        self._inputs.append(self._fields_container2)

    @property
    def fields_container1(self):
        """Allows to connect fields_container1 input to the operator

        Parameters
        ----------
        my_fields_container1 : FieldsContainer, Field, float, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.add_fc()
        >>> op.inputs.fields_container1.connect(my_fields_container1)
        >>> #or
        >>> op.inputs.fields_container1(my_fields_container1)

        """
        return self._fields_container1

    @property
    def fields_container2(self):
        """Allows to connect fields_container2 input to the operator

        Parameters
        ----------
        my_fields_container2 : FieldsContainer, Field, float, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.add_fc()
        >>> op.inputs.fields_container2.connect(my_fields_container2)
        >>> #or
        >>> op.inputs.fields_container2(my_fields_container2)

        """
        return self._fields_container2

class OutputsAddFc(_Outputs):
    """Intermediate class used to get outputs from add_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.add_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(add_fc._spec().outputs, op)
        self._fields_container = Output(add_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.add_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
serialize_to_hdf5
=================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Hdf5 plugin, from "serialization" category
"""

class serialize_to_hdf5(Operator):
    """Serialize the inputs in an hdf5 format.

      available inputs:
        - file_path (str)
        - export_floats (bool)
        - export_flat_vectors (bool)
        - data1 (Any)
        - data2 (Any)

      available outputs:


      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.serialization.serialize_to_hdf5()

      >>> # Make input connections
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_export_floats = bool()
      >>> op.inputs.export_floats.connect(my_export_floats)
      >>> my_export_flat_vectors = bool()
      >>> op.inputs.export_flat_vectors.connect(my_export_flat_vectors)
      >>> my_data1 = dpf.Any()
      >>> op.inputs.data1.connect(my_data1)
      >>> my_data2 = dpf.Any()
      >>> op.inputs.data2.connect(my_data2)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.serialization.serialize_to_hdf5(file_path=my_file_path,export_floats=my_export_floats,export_flat_vectors=my_export_flat_vectors,data1=my_data1,data2=my_data2)

      >>> # Get output data"""
    def __init__(self, file_path=None, export_floats=None, export_flat_vectors=None, data1=None, data2=None, config=None, server=None):
        super().__init__(name="serialize_to_hdf5", config = config, server = server)
        self._inputs = InputsSerializeToHdf5(self)
        self._outputs = OutputsSerializeToHdf5(self)
        if file_path !=None:
            self.inputs.file_path.connect(file_path)
        if export_floats !=None:
            self.inputs.export_floats.connect(export_floats)
        if export_flat_vectors !=None:
            self.inputs.export_flat_vectors.connect(export_flat_vectors)
        if data1 !=None:
            self.inputs.data1.connect(data1)
        if data2 !=None:
            self.inputs.data2.connect(data2)

    @staticmethod
    def _spec():
        spec = Specification(description="""Serialize the inputs in an hdf5 format.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "file_path", type_names=["string"], optional=False, document="""output file path with .h5 extension"""), 
                                 1 : PinSpecification(name = "export_floats", type_names=["bool"], optional=False, document="""converts double to float to reduce file size (default is true)"""), 
                                 2 : PinSpecification(name = "export_flat_vectors", type_names=["bool"], optional=False, document="""if true, vectors and matrices data are exported flat (x1,y1,z1,x2,y2,z2..) (default is false)"""), 
                                 3 : PinSpecification(name = "data", type_names=["any"], optional=False, document="""only the data set explicitly to export is exported"""), 
                                 4 : PinSpecification(name = "data", type_names=["any"], optional=False, document="""only the data set explicitly to export is exported""")},
                             map_output_pin_spec={
})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "serialize_to_hdf5")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSerializeToHdf5 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsSerializeToHdf5 
        """
        return super().outputs


#internal name: serialize_to_hdf5
#scripting name: serialize_to_hdf5
class InputsSerializeToHdf5(_Inputs):
    """Intermediate class used to connect user inputs to serialize_to_hdf5 operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.serialize_to_hdf5()
      >>> my_file_path = str()
      >>> op.inputs.file_path.connect(my_file_path)
      >>> my_export_floats = bool()
      >>> op.inputs.export_floats.connect(my_export_floats)
      >>> my_export_flat_vectors = bool()
      >>> op.inputs.export_flat_vectors.connect(my_export_flat_vectors)
      >>> my_data1 = dpf.Any()
      >>> op.inputs.data1.connect(my_data1)
      >>> my_data2 = dpf.Any()
      >>> op.inputs.data2.connect(my_data2)
    """
    def __init__(self, op: Operator):
        super().__init__(serialize_to_hdf5._spec().inputs, op)
        self._file_path = Input(serialize_to_hdf5._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._file_path)
        self._export_floats = Input(serialize_to_hdf5._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._export_floats)
        self._export_flat_vectors = Input(serialize_to_hdf5._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._export_flat_vectors)
        self._data1 = Input(serialize_to_hdf5._spec().input_pin(3), 3, op, 0) 
        self._inputs.append(self._data1)
        self._data2 = Input(serialize_to_hdf5._spec().input_pin(4), 4, op, 1) 
        self._inputs.append(self._data2)

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator

        - pindoc: output file path with .h5 extension

        Parameters
        ----------
        my_file_path : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> #or
        >>> op.inputs.file_path(my_file_path)

        """
        return self._file_path

    @property
    def export_floats(self):
        """Allows to connect export_floats input to the operator

        - pindoc: converts double to float to reduce file size (default is true)

        Parameters
        ----------
        my_export_floats : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.export_floats.connect(my_export_floats)
        >>> #or
        >>> op.inputs.export_floats(my_export_floats)

        """
        return self._export_floats

    @property
    def export_flat_vectors(self):
        """Allows to connect export_flat_vectors input to the operator

        - pindoc: if true, vectors and matrices data are exported flat (x1,y1,z1,x2,y2,z2..) (default is false)

        Parameters
        ----------
        my_export_flat_vectors : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.export_flat_vectors.connect(my_export_flat_vectors)
        >>> #or
        >>> op.inputs.export_flat_vectors(my_export_flat_vectors)

        """
        return self._export_flat_vectors

    @property
    def data1(self):
        """Allows to connect data1 input to the operator

        - pindoc: only the data set explicitly to export is exported

        Parameters
        ----------
        my_data1 : Any, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.data1.connect(my_data1)
        >>> #or
        >>> op.inputs.data1(my_data1)

        """
        return self._data1

    @property
    def data2(self):
        """Allows to connect data2 input to the operator

        - pindoc: only the data set explicitly to export is exported

        Parameters
        ----------
        my_data2 : Any, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.data2.connect(my_data2)
        >>> #or
        >>> op.inputs.data2(my_data2)

        """
        return self._data2

class OutputsSerializeToHdf5(_Outputs):
    """Intermediate class used to get outputs from serialize_to_hdf5 operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.serialization.serialize_to_hdf5()
      >>> # Connect inputs : op.inputs. ...
    """
    def __init__(self, op: Operator):
        super().__init__(serialize_to_hdf5._spec().outputs, op)
        pass 




from .elemental_from_mesh import elemental_from_mesh 
from .intersect import intersect 
from .connectivity_ids import connectivity_ids 
from .on_property import on_property 
from .transpose import transpose 
from .rescope_fc import rescope_fc 
from .splitted_on_property_type import splitted_on_property_type 
from .rescope import rescope 
from .on_named_selection import on_named_selection 
from .nodal_from_mesh import nodal_from_mesh 
from .change_fc import change_fc 
from .from_mesh import from_mesh 



import logging


def setup_logger(loglevel="INFO"):
    """Set up the logger.

    Parameters
    ----------
    loglevel : str, optional
        The level of the logger to set up. The default is ``"INFO"``.
    """

    # return existing log if this function has already been called
    if hasattr(setup_logger, "log"):
        setup_logger.log.setLevel(loglevel)
        ch = setup_logger.log.handlers[0]
        ch.setLevel(loglevel)
        return setup_logger.log

    # create logger
    log = logging.getLogger(__name__)
    log.setLevel(loglevel)

    # create console handler and set level to debug
    ch = logging.StreamHandler()
    ch.setLevel(loglevel)

    # create formatter
    formatstr = "%(asctime)s [%(levelname)s] %(name)s: %(message)s"
    formatter = logging.Formatter(formatstr)

    # add formatter to ch
    ch.setFormatter(formatter)

    # add ch to logger
    log.addHandler(ch)

    # make persistent
    setup_logger.log = log

    return log



"""
identical_fc
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "logic" category
"""

class identical_fc(Operator):
    """Check if two fields container are identical.

      available inputs:
        - fields_containerA (FieldsContainer)
        - fields_containerB (FieldsContainer)
        - tolerance (float)
        - small_value (float)

      available outputs:
        - boolean (bool)
        - message (str)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.logic.identical_fc()

      >>> # Make input connections
      >>> my_fields_containerA = dpf.FieldsContainer()
      >>> op.inputs.fields_containerA.connect(my_fields_containerA)
      >>> my_fields_containerB = dpf.FieldsContainer()
      >>> op.inputs.fields_containerB.connect(my_fields_containerB)
      >>> my_tolerance = float()
      >>> op.inputs.tolerance.connect(my_tolerance)
      >>> my_small_value = float()
      >>> op.inputs.small_value.connect(my_small_value)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.logic.identical_fc(fields_containerA=my_fields_containerA,fields_containerB=my_fields_containerB,tolerance=my_tolerance,small_value=my_small_value)

      >>> # Get output data
      >>> result_boolean = op.outputs.boolean()
      >>> result_message = op.outputs.message()"""
    def __init__(self, fields_containerA=None, fields_containerB=None, tolerance=None, small_value=None, config=None, server=None):
        super().__init__(name="AreFieldsIdentical_fc", config = config, server = server)
        self._inputs = InputsIdenticalFc(self)
        self._outputs = OutputsIdenticalFc(self)
        if fields_containerA !=None:
            self.inputs.fields_containerA.connect(fields_containerA)
        if fields_containerB !=None:
            self.inputs.fields_containerB.connect(fields_containerB)
        if tolerance !=None:
            self.inputs.tolerance.connect(tolerance)
        if small_value !=None:
            self.inputs.small_value.connect(small_value)

    @staticmethod
    def _spec():
        spec = Specification(description="""Check if two fields container are identical.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_containerA", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "fields_containerB", type_names=["fields_container"], optional=False, document=""""""), 
                                 2 : PinSpecification(name = "tolerance", type_names=["double"], optional=False, document="""Double relative tolerance. Maximum tolerance gap between to compared values: values within relative tolerance are considered identical (v1-v2)/v2 < relativeTol (default is 0.001)."""), 
                                 3 : PinSpecification(name = "small_value", type_names=["double"], optional=False, document="""Double positive small value.Smallest value which will be considered during the comparison step : all the abs(values) in field less than this value is considered as null, (default value:1.0e-14).""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "boolean", type_names=["bool"], optional=False, document="""bool (true if identical...)"""), 
                                 1 : PinSpecification(name = "message", type_names=["string"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "AreFieldsIdentical_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIdenticalFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsIdenticalFc 
        """
        return super().outputs


#internal name: AreFieldsIdentical_fc
#scripting name: identical_fc
class InputsIdenticalFc(_Inputs):
    """Intermediate class used to connect user inputs to identical_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.identical_fc()
      >>> my_fields_containerA = dpf.FieldsContainer()
      >>> op.inputs.fields_containerA.connect(my_fields_containerA)
      >>> my_fields_containerB = dpf.FieldsContainer()
      >>> op.inputs.fields_containerB.connect(my_fields_containerB)
      >>> my_tolerance = float()
      >>> op.inputs.tolerance.connect(my_tolerance)
      >>> my_small_value = float()
      >>> op.inputs.small_value.connect(my_small_value)
    """
    def __init__(self, op: Operator):
        super().__init__(identical_fc._spec().inputs, op)
        self._fields_containerA = Input(identical_fc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_containerA)
        self._fields_containerB = Input(identical_fc._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._fields_containerB)
        self._tolerance = Input(identical_fc._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._tolerance)
        self._small_value = Input(identical_fc._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._small_value)

    @property
    def fields_containerA(self):
        """Allows to connect fields_containerA input to the operator

        Parameters
        ----------
        my_fields_containerA : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_fc()
        >>> op.inputs.fields_containerA.connect(my_fields_containerA)
        >>> #or
        >>> op.inputs.fields_containerA(my_fields_containerA)

        """
        return self._fields_containerA

    @property
    def fields_containerB(self):
        """Allows to connect fields_containerB input to the operator

        Parameters
        ----------
        my_fields_containerB : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_fc()
        >>> op.inputs.fields_containerB.connect(my_fields_containerB)
        >>> #or
        >>> op.inputs.fields_containerB(my_fields_containerB)

        """
        return self._fields_containerB

    @property
    def tolerance(self):
        """Allows to connect tolerance input to the operator

        - pindoc: Double relative tolerance. Maximum tolerance gap between to compared values: values within relative tolerance are considered identical (v1-v2)/v2 < relativeTol (default is 0.001).

        Parameters
        ----------
        my_tolerance : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_fc()
        >>> op.inputs.tolerance.connect(my_tolerance)
        >>> #or
        >>> op.inputs.tolerance(my_tolerance)

        """
        return self._tolerance

    @property
    def small_value(self):
        """Allows to connect small_value input to the operator

        - pindoc: Double positive small value.Smallest value which will be considered during the comparison step : all the abs(values) in field less than this value is considered as null, (default value:1.0e-14).

        Parameters
        ----------
        my_small_value : float, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_fc()
        >>> op.inputs.small_value.connect(my_small_value)
        >>> #or
        >>> op.inputs.small_value(my_small_value)

        """
        return self._small_value

class OutputsIdenticalFc(_Outputs):
    """Intermediate class used to get outputs from identical_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.logic.identical_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_boolean = op.outputs.boolean()
      >>> result_message = op.outputs.message()
    """
    def __init__(self, op: Operator):
        super().__init__(identical_fc._spec().outputs, op)
        self._boolean = Output(identical_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._boolean)
        self._message = Output(identical_fc._spec().output_pin(1), 1, op) 
        self._outputs.append(self._message)

    @property
    def boolean(self):
        """Allows to get boolean output of the operator


        - pindoc: bool (true if identical...)

        Returns
        ----------
        my_boolean : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_boolean = op.outputs.boolean() 
        """
        return self._boolean

    @property
    def message(self):
        """Allows to get message output of the operator


        Returns
        ----------
        my_message : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.logic.identical_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_message = op.outputs.message() 
        """
        return self._message




"""
bind_support
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "utility" category
"""

class bind_support(Operator):
    """Tie a support to a field.

      available inputs:
        - field (Field, FieldsContainer)
        - support (MeshedRegion, AbstractFieldSupport)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.utility.bind_support()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_support = dpf.MeshedRegion()
      >>> op.inputs.support.connect(my_support)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.utility.bind_support(field=my_field,support=my_support)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, field=None, support=None, config=None, server=None):
        super().__init__(name="BindSupport", config = config, server = server)
        self._inputs = InputsBindSupport(self)
        self._outputs = OutputsBindSupport(self)
        if field !=None:
            self.inputs.field.connect(field)
        if support !=None:
            self.inputs.support.connect(support)

    @staticmethod
    def _spec():
        spec = Specification(description="""Tie a support to a field.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected"""), 
                                 1 : PinSpecification(name = "support", type_names=["abstract_meshed_region","abstract_field_support"], optional=False, document="""meshed region or a support of the field""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "BindSupport")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsBindSupport 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsBindSupport 
        """
        return super().outputs


#internal name: BindSupport
#scripting name: bind_support
class InputsBindSupport(_Inputs):
    """Intermediate class used to connect user inputs to bind_support operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.bind_support()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_support = dpf.MeshedRegion()
      >>> op.inputs.support.connect(my_support)
    """
    def __init__(self, op: Operator):
        super().__init__(bind_support._spec().inputs, op)
        self._field = Input(bind_support._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._support = Input(bind_support._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._support)

    @property
    def field(self):
        """Allows to connect field input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_field : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.bind_support()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def support(self):
        """Allows to connect support input to the operator

        - pindoc: meshed region or a support of the field

        Parameters
        ----------
        my_support : MeshedRegion, AbstractFieldSupport, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.bind_support()
        >>> op.inputs.support.connect(my_support)
        >>> #or
        >>> op.inputs.support(my_support)

        """
        return self._support

class OutputsBindSupport(_Outputs):
    """Intermediate class used to get outputs from bind_support operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.utility.bind_support()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(bind_support._spec().outputs, op)
        self._field = Output(bind_support._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.utility.bind_support()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
cplx_dot
========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "math" category
"""

class cplx_dot(Operator):
    """Computes product between two field containers containing complex fields.

      available inputs:
        - fields_containerA (FieldsContainer)
        - fields_containerB (FieldsContainer)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.math.cplx_dot()

      >>> # Make input connections
      >>> my_fields_containerA = dpf.FieldsContainer()
      >>> op.inputs.fields_containerA.connect(my_fields_containerA)
      >>> my_fields_containerB = dpf.FieldsContainer()
      >>> op.inputs.fields_containerB.connect(my_fields_containerB)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.math.cplx_dot(fields_containerA=my_fields_containerA,fields_containerB=my_fields_containerB)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_containerA=None, fields_containerB=None, config=None, server=None):
        super().__init__(name="cplx_dot", config = config, server = server)
        self._inputs = InputsCplxDot(self)
        self._outputs = OutputsCplxDot(self)
        if fields_containerA !=None:
            self.inputs.fields_containerA.connect(fields_containerA)
        if fields_containerB !=None:
            self.inputs.fields_containerB.connect(fields_containerB)

    @staticmethod
    def _spec():
        spec = Specification(description="""Computes product between two field containers containing complex fields.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_containerA", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "fields_containerB", type_names=["fields_container"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "cplx_dot")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCplxDot 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsCplxDot 
        """
        return super().outputs


#internal name: cplx_dot
#scripting name: cplx_dot
class InputsCplxDot(_Inputs):
    """Intermediate class used to connect user inputs to cplx_dot operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.cplx_dot()
      >>> my_fields_containerA = dpf.FieldsContainer()
      >>> op.inputs.fields_containerA.connect(my_fields_containerA)
      >>> my_fields_containerB = dpf.FieldsContainer()
      >>> op.inputs.fields_containerB.connect(my_fields_containerB)
    """
    def __init__(self, op: Operator):
        super().__init__(cplx_dot._spec().inputs, op)
        self._fields_containerA = Input(cplx_dot._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_containerA)
        self._fields_containerB = Input(cplx_dot._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._fields_containerB)

    @property
    def fields_containerA(self):
        """Allows to connect fields_containerA input to the operator

        Parameters
        ----------
        my_fields_containerA : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.cplx_dot()
        >>> op.inputs.fields_containerA.connect(my_fields_containerA)
        >>> #or
        >>> op.inputs.fields_containerA(my_fields_containerA)

        """
        return self._fields_containerA

    @property
    def fields_containerB(self):
        """Allows to connect fields_containerB input to the operator

        Parameters
        ----------
        my_fields_containerB : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.cplx_dot()
        >>> op.inputs.fields_containerB.connect(my_fields_containerB)
        >>> #or
        >>> op.inputs.fields_containerB(my_fields_containerB)

        """
        return self._fields_containerB

class OutputsCplxDot(_Outputs):
    """Intermediate class used to get outputs from cplx_dot operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.math.cplx_dot()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(cplx_dot._spec().outputs, op)
        self._fields_container = Output(cplx_dot._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.math.cplx_dot()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
invariants
==========
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "invariant" category
"""

class invariants(Operator):
    """Computes the element-wise invariants of a tensor field.

      available inputs:
        - field (Field)

      available outputs:
        - field_int (Field)
        - field_eqv (Field)
        - field_max_shear (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.invariant.invariants()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.invariant.invariants(field=my_field)

      >>> # Get output data
      >>> result_field_int = op.outputs.field_int()
      >>> result_field_eqv = op.outputs.field_eqv()
      >>> result_field_max_shear = op.outputs.field_max_shear()"""
    def __init__(self, field=None, config=None, server=None):
        super().__init__(name="invariants_deriv", config = config, server = server)
        self._inputs = InputsInvariants(self)
        self._outputs = OutputsInvariants(self)
        if field !=None:
            self.inputs.field.connect(field)

    @staticmethod
    def _spec():
        spec = Specification(description="""Computes the element-wise invariants of a tensor field.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field_int", type_names=["field"], optional=False, document="""stress intensity field"""), 
                                 1 : PinSpecification(name = "field_eqv", type_names=["field"], optional=False, document="""stress equivalent intensity"""), 
                                 2 : PinSpecification(name = "field_max_shear", type_names=["field"], optional=False, document="""max shear stress field""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "invariants_deriv")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsInvariants 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsInvariants 
        """
        return super().outputs


#internal name: invariants_deriv
#scripting name: invariants
class InputsInvariants(_Inputs):
    """Intermediate class used to connect user inputs to invariants operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.invariant.invariants()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
    """
    def __init__(self, op: Operator):
        super().__init__(invariants._spec().inputs, op)
        self._field = Input(invariants._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)

    @property
    def field(self):
        """Allows to connect field input to the operator

        Parameters
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.invariant.invariants()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

class OutputsInvariants(_Outputs):
    """Intermediate class used to get outputs from invariants operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.invariant.invariants()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field_int = op.outputs.field_int()
      >>> result_field_eqv = op.outputs.field_eqv()
      >>> result_field_max_shear = op.outputs.field_max_shear()
    """
    def __init__(self, op: Operator):
        super().__init__(invariants._spec().outputs, op)
        self._field_int = Output(invariants._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field_int)
        self._field_eqv = Output(invariants._spec().output_pin(1), 1, op) 
        self._outputs.append(self._field_eqv)
        self._field_max_shear = Output(invariants._spec().output_pin(2), 2, op) 
        self._outputs.append(self._field_max_shear)

    @property
    def field_int(self):
        """Allows to get field_int output of the operator


        - pindoc: stress intensity field

        Returns
        ----------
        my_field_int : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.invariant.invariants()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_int = op.outputs.field_int() 
        """
        return self._field_int

    @property
    def field_eqv(self):
        """Allows to get field_eqv output of the operator


        - pindoc: stress equivalent intensity

        Returns
        ----------
        my_field_eqv : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.invariant.invariants()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_eqv = op.outputs.field_eqv() 
        """
        return self._field_eqv

    @property
    def field_max_shear(self):
        """Allows to get field_max_shear output of the operator


        - pindoc: max shear stress field

        Returns
        ----------
        my_field_max_shear : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.invariant.invariants()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field_max_shear = op.outputs.field_max_shear() 
        """
        return self._field_max_shear




"""
from_scoping
============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "mesh" category
"""

class from_scoping(Operator):
    """Extracts a meshed region from an other meshed region base on a scoping

      available inputs:
        - scoping (Scoping)
        - inclusive (int) (optional)
        - mesh (MeshedRegion)

      available outputs:
        - mesh (MeshedRegion)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.mesh.from_scoping()

      >>> # Make input connections
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_inclusive = int()
      >>> op.inputs.inclusive.connect(my_inclusive)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.mesh.from_scoping(scoping=my_scoping,inclusive=my_inclusive,mesh=my_mesh)

      >>> # Get output data
      >>> result_mesh = op.outputs.mesh()"""
    def __init__(self, scoping=None, inclusive=None, mesh=None, config=None, server=None):
        super().__init__(name="mesh::by_scoping", config = config, server = server)
        self._inputs = InputsFromScoping(self)
        self._outputs = OutputsFromScoping(self)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)
        if inclusive !=None:
            self.inputs.inclusive.connect(inclusive)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Extracts a meshed region from an other meshed region base on a scoping""",
                             map_input_pin_spec={
                                 1 : PinSpecification(name = "scoping", type_names=["scoping"], optional=False, document="""if nodal scoping, then the scoping is transposed respecting the inclusive pin"""), 
                                 2 : PinSpecification(name = "inclusive", type_names=["int32"], optional=True, document="""if inclusive == 1 then all the elements adjacent to the nodes ids in input are added, if inclusive == 0, only the elements which have all their nodes in the scoping are included"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mesh::by_scoping")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFromScoping 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsFromScoping 
        """
        return super().outputs


#internal name: mesh::by_scoping
#scripting name: from_scoping
class InputsFromScoping(_Inputs):
    """Intermediate class used to connect user inputs to from_scoping operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.from_scoping()
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_inclusive = int()
      >>> op.inputs.inclusive.connect(my_inclusive)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(from_scoping._spec().inputs, op)
        self._scoping = Input(from_scoping._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._scoping)
        self._inclusive = Input(from_scoping._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._inclusive)
        self._mesh = Input(from_scoping._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: if nodal scoping, then the scoping is transposed respecting the inclusive pin

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.from_scoping()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

    @property
    def inclusive(self):
        """Allows to connect inclusive input to the operator

        - pindoc: if inclusive == 1 then all the elements adjacent to the nodes ids in input are added, if inclusive == 0, only the elements which have all their nodes in the scoping are included

        Parameters
        ----------
        my_inclusive : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.from_scoping()
        >>> op.inputs.inclusive.connect(my_inclusive)
        >>> #or
        >>> op.inputs.inclusive(my_inclusive)

        """
        return self._inclusive

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.from_scoping()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsFromScoping(_Outputs):
    """Intermediate class used to get outputs from from_scoping operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.mesh.from_scoping()
      >>> # Connect inputs : op.inputs. ...
      >>> result_mesh = op.outputs.mesh()
    """
    def __init__(self, op: Operator):
        super().__init__(from_scoping._spec().outputs, op)
        self._mesh = Output(from_scoping._spec().output_pin(0), 0, op) 
        self._outputs.append(self._mesh)

    @property
    def mesh(self):
        """Allows to get mesh output of the operator


        Returns
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.mesh.from_scoping()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh = op.outputs.mesh() 
        """
        return self._mesh




"""
cyclic_support_provider
=======================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from mapdlOperatorsCore plugin, from "metadata" category
"""

class cyclic_support_provider(Operator):
    """Read the cyclic support (DPF entity containing necessary informations for expansions) and expands the mesh.

      available inputs:
        - streams_container (StreamsContainer) (optional)
        - data_sources (DataSources)
        - sector_meshed_region (MeshedRegion, MeshesContainer) (optional)
        - expanded_meshed_region (MeshedRegion, MeshesContainer) (optional)
        - sectors_to_expand (Scoping, ScopingsContainer, list) (optional)

      available outputs:
        - cyclic_support (CyclicSupport)
        - sector_meshes (MeshesContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.metadata.cyclic_support_provider()

      >>> # Make input connections
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_sector_meshed_region = dpf.MeshedRegion()
      >>> op.inputs.sector_meshed_region.connect(my_sector_meshed_region)
      >>> my_expanded_meshed_region = dpf.MeshedRegion()
      >>> op.inputs.expanded_meshed_region.connect(my_expanded_meshed_region)
      >>> my_sectors_to_expand = dpf.Scoping()
      >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.metadata.cyclic_support_provider(streams_container=my_streams_container,data_sources=my_data_sources,sector_meshed_region=my_sector_meshed_region,expanded_meshed_region=my_expanded_meshed_region,sectors_to_expand=my_sectors_to_expand)

      >>> # Get output data
      >>> result_cyclic_support = op.outputs.cyclic_support()
      >>> result_sector_meshes = op.outputs.sector_meshes()"""
    def __init__(self, streams_container=None, data_sources=None, sector_meshed_region=None, expanded_meshed_region=None, sectors_to_expand=None, config=None, server=None):
        super().__init__(name="mapdl::rst::support_provider_cyclic", config = config, server = server)
        self._inputs = InputsCyclicSupportProvider(self)
        self._outputs = OutputsCyclicSupportProvider(self)
        if streams_container !=None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)
        if sector_meshed_region !=None:
            self.inputs.sector_meshed_region.connect(sector_meshed_region)
        if expanded_meshed_region !=None:
            self.inputs.expanded_meshed_region.connect(expanded_meshed_region)
        if sectors_to_expand !=None:
            self.inputs.sectors_to_expand.connect(sectors_to_expand)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read the cyclic support (DPF entity containing necessary informations for expansions) and expands the mesh.""",
                             map_input_pin_spec={
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=True, document="""Streams containing the result file."""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""data sources containing the result file."""), 
                                 7 : PinSpecification(name = "sector_meshed_region", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""mesh of the first sector."""), 
                                 15 : PinSpecification(name = "expanded_meshed_region", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""if this pin is set, expanding the mesh is not necessary."""), 
                                 18 : PinSpecification(name = "sectors_to_expand", type_names=["scoping","scopings_container","vector<int32>"], optional=True, document="""sectors to expand (start at 0), for multistage: use scopings container with 'stage' label.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "cyclic_support", type_names=["cyclic_support"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "sector_meshes", type_names=["meshes_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "mapdl::rst::support_provider_cyclic")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsCyclicSupportProvider 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsCyclicSupportProvider 
        """
        return super().outputs


#internal name: mapdl::rst::support_provider_cyclic
#scripting name: cyclic_support_provider
class InputsCyclicSupportProvider(_Inputs):
    """Intermediate class used to connect user inputs to cyclic_support_provider operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.metadata.cyclic_support_provider()
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_sector_meshed_region = dpf.MeshedRegion()
      >>> op.inputs.sector_meshed_region.connect(my_sector_meshed_region)
      >>> my_expanded_meshed_region = dpf.MeshedRegion()
      >>> op.inputs.expanded_meshed_region.connect(my_expanded_meshed_region)
      >>> my_sectors_to_expand = dpf.Scoping()
      >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
    """
    def __init__(self, op: Operator):
        super().__init__(cyclic_support_provider._spec().inputs, op)
        self._streams_container = Input(cyclic_support_provider._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(cyclic_support_provider._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)
        self._sector_meshed_region = Input(cyclic_support_provider._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._sector_meshed_region)
        self._expanded_meshed_region = Input(cyclic_support_provider._spec().input_pin(15), 15, op, -1) 
        self._inputs.append(self._expanded_meshed_region)
        self._sectors_to_expand = Input(cyclic_support_provider._spec().input_pin(18), 18, op, -1) 
        self._inputs.append(self._sectors_to_expand)

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        - pindoc: Streams containing the result file.

        Parameters
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: data sources containing the result file.

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

    @property
    def sector_meshed_region(self):
        """Allows to connect sector_meshed_region input to the operator

        - pindoc: mesh of the first sector.

        Parameters
        ----------
        my_sector_meshed_region : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> op.inputs.sector_meshed_region.connect(my_sector_meshed_region)
        >>> #or
        >>> op.inputs.sector_meshed_region(my_sector_meshed_region)

        """
        return self._sector_meshed_region

    @property
    def expanded_meshed_region(self):
        """Allows to connect expanded_meshed_region input to the operator

        - pindoc: if this pin is set, expanding the mesh is not necessary.

        Parameters
        ----------
        my_expanded_meshed_region : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> op.inputs.expanded_meshed_region.connect(my_expanded_meshed_region)
        >>> #or
        >>> op.inputs.expanded_meshed_region(my_expanded_meshed_region)

        """
        return self._expanded_meshed_region

    @property
    def sectors_to_expand(self):
        """Allows to connect sectors_to_expand input to the operator

        - pindoc: sectors to expand (start at 0), for multistage: use scopings container with 'stage' label.

        Parameters
        ----------
        my_sectors_to_expand : Scoping, ScopingsContainer, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> op.inputs.sectors_to_expand.connect(my_sectors_to_expand)
        >>> #or
        >>> op.inputs.sectors_to_expand(my_sectors_to_expand)

        """
        return self._sectors_to_expand

class OutputsCyclicSupportProvider(_Outputs):
    """Intermediate class used to get outputs from cyclic_support_provider operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.metadata.cyclic_support_provider()
      >>> # Connect inputs : op.inputs. ...
      >>> result_cyclic_support = op.outputs.cyclic_support()
      >>> result_sector_meshes = op.outputs.sector_meshes()
    """
    def __init__(self, op: Operator):
        super().__init__(cyclic_support_provider._spec().outputs, op)
        self._cyclic_support = Output(cyclic_support_provider._spec().output_pin(0), 0, op) 
        self._outputs.append(self._cyclic_support)
        self._sector_meshes = Output(cyclic_support_provider._spec().output_pin(1), 1, op) 
        self._outputs.append(self._sector_meshes)

    @property
    def cyclic_support(self):
        """Allows to get cyclic_support output of the operator


        Returns
        ----------
        my_cyclic_support : CyclicSupport, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_cyclic_support = op.outputs.cyclic_support() 
        """
        return self._cyclic_support

    @property
    def sector_meshes(self):
        """Allows to get sector_meshes output of the operator


        Returns
        ----------
        my_sector_meshes : MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.metadata.cyclic_support_provider()
        >>> # Connect inputs : op.inputs. ...
        >>> result_sector_meshes = op.outputs.sector_meshes() 
        """
        return self._sector_meshes




from .normals_provider_nl import normals_provider_nl 
from .rotate_in_cylindrical_cs_fc import rotate_in_cylindrical_cs_fc 
from .rotate_in_cylindrical_cs import rotate_in_cylindrical_cs 
from .rotate import rotate 
from .rotate_fc import rotate_fc 
from .to_polar_coordinates import to_polar_coordinates 
from .elements_volumes_over_time import elements_volumes_over_time 
from .gauss_to_node import gauss_to_node 
from .elements_facets_surfaces_over_time import elements_facets_surfaces_over_time 
from .elements_volume import elements_volume 
from .element_nodal_contribution import element_nodal_contribution 
from .center_of_gravity import center_of_gravity 
from .integrate_over_elements import integrate_over_elements 
from .mass import mass 
from .moment_of_inertia import moment_of_inertia 
from .normals import normals 



"""
max_by_component
================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "min_max" category
"""

class max_by_component(Operator):
    """Give the maximum for each element rank by comparing several fields.

      available inputs:
        - use_absolute_value (bool)
        - fieldA1 (Field, FieldsContainer)
        - fieldA2 (Field, FieldsContainer)
        - fieldB2 (Field, FieldsContainer)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.min_max.max_by_component()

      >>> # Make input connections
      >>> my_use_absolute_value = bool()
      >>> op.inputs.use_absolute_value.connect(my_use_absolute_value)
      >>> my_fieldA1 = dpf.Field()
      >>> op.inputs.fieldA1.connect(my_fieldA1)
      >>> my_fieldA2 = dpf.Field()
      >>> op.inputs.fieldA2.connect(my_fieldA2)
      >>> my_fieldB2 = dpf.Field()
      >>> op.inputs.fieldB2.connect(my_fieldB2)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.min_max.max_by_component(use_absolute_value=my_use_absolute_value,fieldA1=my_fieldA1,fieldA2=my_fieldA2,fieldB2=my_fieldB2)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, use_absolute_value=None, fieldA1=None, fieldA2=None, fieldB2=None, config=None, server=None):
        super().__init__(name="max_by_component", config = config, server = server)
        self._inputs = InputsMaxByComponent(self)
        self._outputs = OutputsMaxByComponent(self)
        if use_absolute_value !=None:
            self.inputs.use_absolute_value.connect(use_absolute_value)
        if fieldA1 !=None:
            self.inputs.fieldA1.connect(fieldA1)
        if fieldA2 !=None:
            self.inputs.fieldA2.connect(fieldA2)
        if fieldB2 !=None:
            self.inputs.fieldB2.connect(fieldB2)

    @staticmethod
    def _spec():
        spec = Specification(description="""Give the maximum for each element rank by comparing several fields.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "use_absolute_value", type_names=["bool"], optional=False, document="""use_absolute_value"""), 
                                 1 : PinSpecification(name = "fieldA", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected"""), 
                                 2 : PinSpecification(name = "fieldA", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected"""), 
                                 3 : PinSpecification(name = "fieldB", type_names=["field","fields_container"], optional=False, document="""field or fields container with only one field is expected""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "max_by_component")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMaxByComponent 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMaxByComponent 
        """
        return super().outputs


#internal name: max_by_component
#scripting name: max_by_component
class InputsMaxByComponent(_Inputs):
    """Intermediate class used to connect user inputs to max_by_component operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.max_by_component()
      >>> my_use_absolute_value = bool()
      >>> op.inputs.use_absolute_value.connect(my_use_absolute_value)
      >>> my_fieldA1 = dpf.Field()
      >>> op.inputs.fieldA1.connect(my_fieldA1)
      >>> my_fieldA2 = dpf.Field()
      >>> op.inputs.fieldA2.connect(my_fieldA2)
      >>> my_fieldB2 = dpf.Field()
      >>> op.inputs.fieldB2.connect(my_fieldB2)
    """
    def __init__(self, op: Operator):
        super().__init__(max_by_component._spec().inputs, op)
        self._use_absolute_value = Input(max_by_component._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._use_absolute_value)
        self._fieldA1 = Input(max_by_component._spec().input_pin(1), 1, op, 0) 
        self._inputs.append(self._fieldA1)
        self._fieldA2 = Input(max_by_component._spec().input_pin(2), 2, op, 1) 
        self._inputs.append(self._fieldA2)
        self._fieldB2 = Input(max_by_component._spec().input_pin(3), 3, op, 1) 
        self._inputs.append(self._fieldB2)

    @property
    def use_absolute_value(self):
        """Allows to connect use_absolute_value input to the operator

        - pindoc: use_absolute_value

        Parameters
        ----------
        my_use_absolute_value : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.use_absolute_value.connect(my_use_absolute_value)
        >>> #or
        >>> op.inputs.use_absolute_value(my_use_absolute_value)

        """
        return self._use_absolute_value

    @property
    def fieldA1(self):
        """Allows to connect fieldA1 input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_fieldA1 : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.fieldA1.connect(my_fieldA1)
        >>> #or
        >>> op.inputs.fieldA1(my_fieldA1)

        """
        return self._fieldA1

    @property
    def fieldA2(self):
        """Allows to connect fieldA2 input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_fieldA2 : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.fieldA2.connect(my_fieldA2)
        >>> #or
        >>> op.inputs.fieldA2(my_fieldA2)

        """
        return self._fieldA2

    @property
    def fieldB2(self):
        """Allows to connect fieldB2 input to the operator

        - pindoc: field or fields container with only one field is expected

        Parameters
        ----------
        my_fieldB2 : Field, FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.max_by_component()
        >>> op.inputs.fieldB2.connect(my_fieldB2)
        >>> #or
        >>> op.inputs.fieldB2(my_fieldB2)

        """
        return self._fieldB2

class OutputsMaxByComponent(_Outputs):
    """Intermediate class used to get outputs from max_by_component operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.max_by_component()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(max_by_component._spec().outputs, op)
        self._field = Output(max_by_component._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.max_by_component()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




"""
elemental_difference_fc
=======================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "averaging" category
"""

class elemental_difference_fc(Operator):
    """Transform ElementalNodal or Nodal field into Elemental field. Each elemental value is the maximum difference between the unaveraged or averaged (depending on the input fields) computed result for all nodes in this element. Result is computed on a given element scoping. If the input fields are mixed shell/solid and the shells layers are not asked to be collapsed, then the fields are splitted by element shape and the output fields container has elshape label.

      available inputs:
        - fields_container (FieldsContainer)
        - mesh (MeshedRegion, MeshesContainer) (optional)
        - scoping (Scoping, ScopingsContainer) (optional)
        - collapse_shell_layers (bool) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.averaging.elemental_difference_fc()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_collapse_shell_layers = bool()
      >>> op.inputs.collapse_shell_layers.connect(my_collapse_shell_layers)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.averaging.elemental_difference_fc(fields_container=my_fields_container,mesh=my_mesh,scoping=my_scoping,collapse_shell_layers=my_collapse_shell_layers)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, mesh=None, scoping=None, collapse_shell_layers=None, config=None, server=None):
        super().__init__(name="elemental_difference_fc", config = config, server = server)
        self._inputs = InputsElementalDifferenceFc(self)
        self._outputs = OutputsElementalDifferenceFc(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)
        if collapse_shell_layers !=None:
            self.inputs.collapse_shell_layers.connect(collapse_shell_layers)

    @staticmethod
    def _spec():
        spec = Specification(description="""Transform ElementalNodal or Nodal field into Elemental field. Each elemental value is the maximum difference between the unaveraged or averaged (depending on the input fields) computed result for all nodes in this element. Result is computed on a given element scoping. If the input fields are mixed shell/solid and the shells layers are not asked to be collapsed, then the fields are splitted by element shape and the output fields container has elshape label.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""the mesh region in this pin is used to perform the averaging, if there is no field's support it is used"""), 
                                 3 : PinSpecification(name = "scoping", type_names=["scoping","scopings_container"], optional=True, document="""average only on these elements, if it is scoping container, the label must correspond to the one of the fields container"""), 
                                 10 : PinSpecification(name = "collapse_shell_layers", type_names=["bool"], optional=True, document="""the max elemental difference is taken through the different shell layers if true (default is false)""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "elemental_difference_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementalDifferenceFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsElementalDifferenceFc 
        """
        return super().outputs


#internal name: elemental_difference_fc
#scripting name: elemental_difference_fc
class InputsElementalDifferenceFc(_Inputs):
    """Intermediate class used to connect user inputs to elemental_difference_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.elemental_difference_fc()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_collapse_shell_layers = bool()
      >>> op.inputs.collapse_shell_layers.connect(my_collapse_shell_layers)
    """
    def __init__(self, op: Operator):
        super().__init__(elemental_difference_fc._spec().inputs, op)
        self._fields_container = Input(elemental_difference_fc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._mesh = Input(elemental_difference_fc._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)
        self._scoping = Input(elemental_difference_fc._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._scoping)
        self._collapse_shell_layers = Input(elemental_difference_fc._spec().input_pin(10), 10, op, -1) 
        self._inputs.append(self._collapse_shell_layers)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_difference_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: the mesh region in this pin is used to perform the averaging, if there is no field's support it is used

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_difference_fc()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: average only on these elements, if it is scoping container, the label must correspond to the one of the fields container

        Parameters
        ----------
        my_scoping : Scoping, ScopingsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_difference_fc()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

    @property
    def collapse_shell_layers(self):
        """Allows to connect collapse_shell_layers input to the operator

        - pindoc: the max elemental difference is taken through the different shell layers if true (default is false)

        Parameters
        ----------
        my_collapse_shell_layers : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_difference_fc()
        >>> op.inputs.collapse_shell_layers.connect(my_collapse_shell_layers)
        >>> #or
        >>> op.inputs.collapse_shell_layers(my_collapse_shell_layers)

        """
        return self._collapse_shell_layers

class OutputsElementalDifferenceFc(_Outputs):
    """Intermediate class used to get outputs from elemental_difference_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.elemental_difference_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(elemental_difference_fc._spec().outputs, op)
        self._fields_container = Output(elemental_difference_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.elemental_difference_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
gauss_to_node_fc
================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "averaging" category
"""

class gauss_to_node_fc(Operator):
    """Extrapolating results available at Gauss or quadrature points to nodal points for a field container. The available elements are : Linear quadrangle , parabolique quadrangle,Linear Hexagonal, quadratic hexagonal , linear tetrahedral, quadratic tetrahedral 

      available inputs:
        - fields_container (FieldsContainer)
        - mesh (MeshedRegion, MeshesContainer) (optional)
        - scoping (Scoping) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.averaging.gauss_to_node_fc()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.averaging.gauss_to_node_fc(fields_container=my_fields_container,mesh=my_mesh,scoping=my_scoping)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, fields_container=None, mesh=None, scoping=None, config=None, server=None):
        super().__init__(name="gauss_to_node_fc", config = config, server = server)
        self._inputs = InputsGaussToNodeFc(self)
        self._outputs = OutputsGaussToNodeFc(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)

    @staticmethod
    def _spec():
        spec = Specification(description="""Extrapolating results available at Gauss or quadrature points to nodal points for a field container. The available elements are : Linear quadrangle , parabolique quadrangle,Linear Hexagonal, quadratic hexagonal , linear tetrahedral, quadratic tetrahedral """,
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""the mesh region in this pin is used for Extrapolating results available at Gauss or quadrature points to nodal points."""), 
                                 3 : PinSpecification(name = "scoping", type_names=["scoping"], optional=True, document="""Extrapolating results on the scoping selected by the user, if it is scoping container, the label must correspond to the one of the fields container""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "gauss_to_node_fc")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsGaussToNodeFc 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsGaussToNodeFc 
        """
        return super().outputs


#internal name: gauss_to_node_fc
#scripting name: gauss_to_node_fc
class InputsGaussToNodeFc(_Inputs):
    """Intermediate class used to connect user inputs to gauss_to_node_fc operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.gauss_to_node_fc()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
    """
    def __init__(self, op: Operator):
        super().__init__(gauss_to_node_fc._spec().inputs, op)
        self._fields_container = Input(gauss_to_node_fc._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)
        self._mesh = Input(gauss_to_node_fc._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh)
        self._scoping = Input(gauss_to_node_fc._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._scoping)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.gauss_to_node_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: the mesh region in this pin is used for Extrapolating results available at Gauss or quadrature points to nodal points.

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.gauss_to_node_fc()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: Extrapolating results on the scoping selected by the user, if it is scoping container, the label must correspond to the one of the fields container

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.gauss_to_node_fc()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

class OutputsGaussToNodeFc(_Outputs):
    """Intermediate class used to get outputs from gauss_to_node_fc operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.averaging.gauss_to_node_fc()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(gauss_to_node_fc._spec().outputs, op)
        self._fields_container = Output(gauss_to_node_fc._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.averaging.gauss_to_node_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
integrate_over_elements
=======================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.FEMutils plugin, from "geo" category
"""

class integrate_over_elements(Operator):
    """Integration of an input field over mesh.

      available inputs:
        - field (Field)
        - scoping (Scoping) (optional)
        - mesh (MeshedRegion) (optional)

      available outputs:
        - field (Field)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.geo.integrate_over_elements()

      >>> # Make input connections
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.geo.integrate_over_elements(field=my_field,scoping=my_scoping,mesh=my_mesh)

      >>> # Get output data
      >>> result_field = op.outputs.field()"""
    def __init__(self, field=None, scoping=None, mesh=None, config=None, server=None):
        super().__init__(name="element::integrate", config = config, server = server)
        self._inputs = InputsIntegrateOverElements(self)
        self._outputs = OutputsIntegrateOverElements(self)
        if field !=None:
            self.inputs.field.connect(field)
        if scoping !=None:
            self.inputs.scoping.connect(scoping)
        if mesh !=None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        spec = Specification(description="""Integration of an input field over mesh.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "scoping", type_names=["scoping"], optional=True, document="""Integrate the input field over a specific scoping."""), 
                                 2 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region"], optional=True, document="""Mesh to integrate on, if not provided the one from input field is provided.""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "field", type_names=["field"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "element::integrate")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIntegrateOverElements 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsIntegrateOverElements 
        """
        return super().outputs


#internal name: element::integrate
#scripting name: integrate_over_elements
class InputsIntegrateOverElements(_Inputs):
    """Intermediate class used to connect user inputs to integrate_over_elements operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.geo.integrate_over_elements()
      >>> my_field = dpf.Field()
      >>> op.inputs.field.connect(my_field)
      >>> my_scoping = dpf.Scoping()
      >>> op.inputs.scoping.connect(my_scoping)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
    """
    def __init__(self, op: Operator):
        super().__init__(integrate_over_elements._spec().inputs, op)
        self._field = Input(integrate_over_elements._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._field)
        self._scoping = Input(integrate_over_elements._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._scoping)
        self._mesh = Input(integrate_over_elements._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._mesh)

    @property
    def field(self):
        """Allows to connect field input to the operator

        Parameters
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.integrate_over_elements()
        >>> op.inputs.field.connect(my_field)
        >>> #or
        >>> op.inputs.field(my_field)

        """
        return self._field

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator

        - pindoc: Integrate the input field over a specific scoping.

        Parameters
        ----------
        my_scoping : Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.integrate_over_elements()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> #or
        >>> op.inputs.scoping(my_scoping)

        """
        return self._scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: Mesh to integrate on, if not provided the one from input field is provided.

        Parameters
        ----------
        my_mesh : MeshedRegion, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.integrate_over_elements()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

class OutputsIntegrateOverElements(_Outputs):
    """Intermediate class used to get outputs from integrate_over_elements operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.geo.integrate_over_elements()
      >>> # Connect inputs : op.inputs. ...
      >>> result_field = op.outputs.field()
    """
    def __init__(self, op: Operator):
        super().__init__(integrate_over_elements._spec().outputs, op)
        self._field = Output(integrate_over_elements._spec().output_pin(0), 0, op) 
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator


        Returns
        ----------
        my_field : Field, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.geo.integrate_over_elements()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field() 
        """
        return self._field




from .mesh_support_provider import mesh_support_provider 
from .result_info_provider import result_info_provider 
from .time_freq_provider import time_freq_provider 
from .material_provider import material_provider 
from .streams_provider import streams_provider 
from .mesh_selection_manager_provider import mesh_selection_manager_provider 
from .boundary_condition_provider import boundary_condition_provider 
from .is_cyclic import is_cyclic 
from .material_support_provider import material_support_provider 
from .cyclic_mesh_expansion import cyclic_mesh_expansion 
from .cyclic_support_provider import cyclic_support_provider 



"""
elastic_strain_XZ
=================
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "result" category
"""

class elastic_strain_XZ(Operator):
    """Read/compute element nodal component elastic strains XZ shear component (02 component) by calling the readers defined by the datasources. Regarding the requested location and the input mesh scoping, the result location can be Nodal/ElementalNodal/Elemental.

      available inputs:
        - time_scoping (Scoping, int, listfloat, Field, list) (optional)
        - mesh_scoping (ScopingsContainer, Scoping) (optional)
        - fields_container (FieldsContainer) (optional)
        - streams_container (StreamsContainer) (optional)
        - data_sources (DataSources)
        - bool_rotate_to_global (bool) (optional)
        - mesh (MeshedRegion, MeshesContainer) (optional)
        - requested_location (str) (optional)
        - read_cyclic (int) (optional)

      available outputs:
        - fields_container (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.result.elastic_strain_XZ()

      >>> # Make input connections
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_bool_rotate_to_global = bool()
      >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_read_cyclic = int()
      >>> op.inputs.read_cyclic.connect(my_read_cyclic)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.result.elastic_strain_XZ(time_scoping=my_time_scoping,mesh_scoping=my_mesh_scoping,data_sources=my_data_sources,requested_location=my_requested_location)

      >>> # Get output data
      >>> result_fields_container = op.outputs.fields_container()"""
    def __init__(self, time_scoping=None, mesh_scoping=None, data_sources=None, requested_location=None, config=None, server=None):
        super().__init__(name="EPELXZ", config = config, server = server)
        self._inputs = InputsElasticStrainXZ(self)
        self._outputs = OutputsElasticStrainXZ(self)
        if time_scoping !=None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping !=None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if data_sources !=None:
            self.inputs.data_sources.connect(data_sources)
        if requested_location !=None:
            self.inputs.requested_location.connect(requested_location)

    @staticmethod
    def _spec():
        spec = Specification(description="""Read/compute element nodal component elastic strains XZ shear component (02 component) by calling the readers defined by the datasources. Regarding the requested location and the input mesh scoping, the result location can be Nodal/ElementalNodal/Elemental.""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "time_scoping", type_names=["scoping","int32","vector<int32>","double","field","vector<double>"], optional=True, document="""time/freq (use doubles or field), time/freq set ids (use ints or scoping) or time/freq step ids (use scoping with TimeFreq_steps location) requiered in output"""), 
                                 1 : PinSpecification(name = "mesh_scoping", type_names=["scopings_container","scoping"], optional=True, document="""nodes or elements scoping requiered in output. The scoping's location indicates whether nodes or elements are asked. Using scopings container enables to split the result fields container in domains"""), 
                                 2 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=True, document="""FieldsContainer already allocated modified inplace"""), 
                                 3 : PinSpecification(name = "streams_container", type_names=["streams_container"], optional=True, document="""result file container allowed to be kept open to cache data"""), 
                                 4 : PinSpecification(name = "data_sources", type_names=["data_sources"], optional=False, document="""result file path container, used if no streams are set"""), 
                                 5 : PinSpecification(name = "bool_rotate_to_global", type_names=["bool"], optional=True, document="""if true the field is rotated to global coordinate system (default true)"""), 
                                 7 : PinSpecification(name = "mesh", type_names=["abstract_meshed_region","meshes_container"], optional=True, document="""prevents from reading the mesh in the result files"""), 
                                 9 : PinSpecification(name = "requested_location", type_names=["string"], optional=True, document="""requested location, default is Nodal"""), 
                                 14 : PinSpecification(name = "read_cyclic", type_names=["int32"], optional=True, document="""if 0 cyclic symmetry is ignored, if 1 cyclic sector is read, if 2 cyclic expansion is done, if 3 cyclic expansion is done and stages are merged (default is 1)""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "EPELXZ")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElasticStrainXZ 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsElasticStrainXZ 
        """
        return super().outputs


#internal name: EPELXZ
#scripting name: elastic_strain_XZ
class InputsElasticStrainXZ(_Inputs):
    """Intermediate class used to connect user inputs to elastic_strain_XZ operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.elastic_strain_XZ()
      >>> my_time_scoping = dpf.Scoping()
      >>> op.inputs.time_scoping.connect(my_time_scoping)
      >>> my_mesh_scoping = dpf.ScopingsContainer()
      >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
      >>> my_streams_container = dpf.StreamsContainer()
      >>> op.inputs.streams_container.connect(my_streams_container)
      >>> my_data_sources = dpf.DataSources()
      >>> op.inputs.data_sources.connect(my_data_sources)
      >>> my_bool_rotate_to_global = bool()
      >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
      >>> my_mesh = dpf.MeshedRegion()
      >>> op.inputs.mesh.connect(my_mesh)
      >>> my_requested_location = str()
      >>> op.inputs.requested_location.connect(my_requested_location)
      >>> my_read_cyclic = int()
      >>> op.inputs.read_cyclic.connect(my_read_cyclic)
    """
    def __init__(self, op: Operator):
        super().__init__(elastic_strain_XZ._spec().inputs, op)
        self._time_scoping = Input(elastic_strain_XZ._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(elastic_strain_XZ._spec().input_pin(1), 1, op, -1) 
        self._inputs.append(self._mesh_scoping)
        self._fields_container = Input(elastic_strain_XZ._spec().input_pin(2), 2, op, -1) 
        self._inputs.append(self._fields_container)
        self._streams_container = Input(elastic_strain_XZ._spec().input_pin(3), 3, op, -1) 
        self._inputs.append(self._streams_container)
        self._data_sources = Input(elastic_strain_XZ._spec().input_pin(4), 4, op, -1) 
        self._inputs.append(self._data_sources)
        self._bool_rotate_to_global = Input(elastic_strain_XZ._spec().input_pin(5), 5, op, -1) 
        self._inputs.append(self._bool_rotate_to_global)
        self._mesh = Input(elastic_strain_XZ._spec().input_pin(7), 7, op, -1) 
        self._inputs.append(self._mesh)
        self._requested_location = Input(elastic_strain_XZ._spec().input_pin(9), 9, op, -1) 
        self._inputs.append(self._requested_location)
        self._read_cyclic = Input(elastic_strain_XZ._spec().input_pin(14), 14, op, -1) 
        self._inputs.append(self._read_cyclic)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator

        - pindoc: time/freq (use doubles or field), time/freq set ids (use ints or scoping) or time/freq step ids (use scoping with TimeFreq_steps location) requiered in output

        Parameters
        ----------
        my_time_scoping : Scoping, int, list, float, Field, list, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> #or
        >>> op.inputs.time_scoping(my_time_scoping)

        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator

        - pindoc: nodes or elements scoping requiered in output. The scoping's location indicates whether nodes or elements are asked. Using scopings container enables to split the result fields container in domains

        Parameters
        ----------
        my_mesh_scoping : ScopingsContainer, Scoping, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> #or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)

        """
        return self._mesh_scoping

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        - pindoc: FieldsContainer already allocated modified inplace

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator

        - pindoc: result file container allowed to be kept open to cache data

        Parameters
        ----------
        my_streams_container : StreamsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> #or
        >>> op.inputs.streams_container(my_streams_container)

        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator

        - pindoc: result file path container, used if no streams are set

        Parameters
        ----------
        my_data_sources : DataSources, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> #or
        >>> op.inputs.data_sources(my_data_sources)

        """
        return self._data_sources

    @property
    def bool_rotate_to_global(self):
        """Allows to connect bool_rotate_to_global input to the operator

        - pindoc: if true the field is rotated to global coordinate system (default true)

        Parameters
        ----------
        my_bool_rotate_to_global : bool, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.bool_rotate_to_global.connect(my_bool_rotate_to_global)
        >>> #or
        >>> op.inputs.bool_rotate_to_global(my_bool_rotate_to_global)

        """
        return self._bool_rotate_to_global

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator

        - pindoc: prevents from reading the mesh in the result files

        Parameters
        ----------
        my_mesh : MeshedRegion, MeshesContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> #or
        >>> op.inputs.mesh(my_mesh)

        """
        return self._mesh

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator

        - pindoc: requested location, default is Nodal

        Parameters
        ----------
        my_requested_location : str, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> #or
        >>> op.inputs.requested_location(my_requested_location)

        """
        return self._requested_location

    @property
    def read_cyclic(self):
        """Allows to connect read_cyclic input to the operator

        - pindoc: if 0 cyclic symmetry is ignored, if 1 cyclic sector is read, if 2 cyclic expansion is done, if 3 cyclic expansion is done and stages are merged (default is 1)

        Parameters
        ----------
        my_read_cyclic : int, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> op.inputs.read_cyclic.connect(my_read_cyclic)
        >>> #or
        >>> op.inputs.read_cyclic(my_read_cyclic)

        """
        return self._read_cyclic

class OutputsElasticStrainXZ(_Outputs):
    """Intermediate class used to get outputs from elastic_strain_XZ operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.result.elastic_strain_XZ()
      >>> # Connect inputs : op.inputs. ...
      >>> result_fields_container = op.outputs.fields_container()
    """
    def __init__(self, op: Operator):
        super().__init__(elastic_strain_XZ._spec().outputs, op)
        self._fields_container = Output(elastic_strain_XZ._spec().output_pin(0), 0, op) 
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator


        Returns
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.result.elastic_strain_XZ()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container() 
        """
        return self._fields_container




"""
min_max_by_time
===============
"""
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs, _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification

"""Operators from Ans.Dpf.Native plugin, from "min_max" category
"""

class min_max_by_time(Operator):
    """Evaluates minimum, maximum by time or frequency over all the entities of each field

      available inputs:
        - fields_container (FieldsContainer)

      available outputs:
        - min (FieldsContainer)
        - max (FieldsContainer)

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> # Instantiate operator
      >>> op = dpf.operators.min_max.min_max_by_time()

      >>> # Make input connections
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)

      >>> # Instantiate operator and connect inputs in one line
      >>> op = dpf.operators.min_max.min_max_by_time(fields_container=my_fields_container)

      >>> # Get output data
      >>> result_min = op.outputs.min()
      >>> result_max = op.outputs.max()"""
    def __init__(self, fields_container=None, config=None, server=None):
        super().__init__(name="min_max_by_time", config = config, server = server)
        self._inputs = InputsMinMaxByTime(self)
        self._outputs = OutputsMinMaxByTime(self)
        if fields_container !=None:
            self.inputs.fields_container.connect(fields_container)

    @staticmethod
    def _spec():
        spec = Specification(description="""Evaluates minimum, maximum by time or frequency over all the entities of each field""",
                             map_input_pin_spec={
                                 0 : PinSpecification(name = "fields_container", type_names=["fields_container"], optional=False, document="""""")},
                             map_output_pin_spec={
                                 0 : PinSpecification(name = "min", type_names=["fields_container"], optional=False, document=""""""), 
                                 1 : PinSpecification(name = "max", type_names=["fields_container"], optional=False, document="""""")})
        return spec


    @staticmethod
    def default_config():
        return Operator.default_config(name = "min_max_by_time")

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMinMaxByTime 
        """
        return super().inputs


    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluationg it

        Returns
        --------
        outputs : OutputsMinMaxByTime 
        """
        return super().outputs


#internal name: min_max_by_time
#scripting name: min_max_by_time
class InputsMinMaxByTime(_Inputs):
    """Intermediate class used to connect user inputs to min_max_by_time operator

      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_max_by_time()
      >>> my_fields_container = dpf.FieldsContainer()
      >>> op.inputs.fields_container.connect(my_fields_container)
    """
    def __init__(self, op: Operator):
        super().__init__(min_max_by_time._spec().inputs, op)
        self._fields_container = Input(min_max_by_time._spec().input_pin(0), 0, op, -1) 
        self._inputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator

        Parameters
        ----------
        my_fields_container : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_by_time()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> #or
        >>> op.inputs.fields_container(my_fields_container)

        """
        return self._fields_container

class OutputsMinMaxByTime(_Outputs):
    """Intermediate class used to get outputs from min_max_by_time operator
      Examples
      --------
      >>> from ansys.dpf import core as dpf

      >>> op = dpf.operators.min_max.min_max_by_time()
      >>> # Connect inputs : op.inputs. ...
      >>> result_min = op.outputs.min()
      >>> result_max = op.outputs.max()
    """
    def __init__(self, op: Operator):
        super().__init__(min_max_by_time._spec().outputs, op)
        self._min = Output(min_max_by_time._spec().output_pin(0), 0, op) 
        self._outputs.append(self._min)
        self._max = Output(min_max_by_time._spec().output_pin(1), 1, op) 
        self._outputs.append(self._max)

    @property
    def min(self):
        """Allows to get min output of the operator


        Returns
        ----------
        my_min : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_by_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_min = op.outputs.min() 
        """
        return self._min

    @property
    def max(self):
        """Allows to get max output of the operator


        Returns
        ----------
        my_max : FieldsContainer, 

        Examples
        --------
        >>> from ansys.dpf import core as dpf

        >>> op = dpf.operators.min_max.min_max_by_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_max = op.outputs.max() 
        """
        return self._max




"""
.. _ref_fields_container:

FieldsContainer
===============
Contains classes associated with the DPF FieldsContainer.
"""
from ansys import dpf
from ansys.dpf.core.collection import Collection
from ansys.dpf.core.common import types
from ansys.dpf.core import errors as dpf_errors


class FieldsContainer(Collection):
    """Represents a fields container, which contains fields belonging to a common result.

    A fields container is a set of fields ordered by labels and IDs. Each field
    of the fields container has an ID for each label defining the given fields
    container. These IDs allow splitting the fields on any criteria.

    The most common fields container has the label ``"time"`` with IDs
    corresponding to time sets. The label ``"complex"``, which is
    used in a harmonic analysis for example, allows real parts (``id=0``)
    to be separated from imaginary parts (``id=1``).

    Parameters
    ----------
    fields_container : ansys.grpc.dpf.collection_pb2.Collection or FieldsContainer, optional
        Fields container created from either a collection message or by copying an existing
        fields container. The default is "None``.
    server : ansys.dpf.core.server, optional
        Server with the channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Examples
    --------
    Extract a displacement fields container from a transient result file.

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)
    >>> disp = model.results.displacement()
    >>> disp.inputs.time_scoping.connect([1,5])
    >>> fields_container = disp.outputs.fields_container()
    >>> field_set_5 =fields_container.get_fields_by_time_complex_ids(5)
    >>> #print(fields_container)


    Create a fields container from scratch.

    >>> from ansys.dpf import core as dpf
    >>> fc= dpf.FieldsContainer()
    >>> fc.labels =['time','complex']
    >>> for i in range(0,20): #real fields
    ...     mscop = {"time":i+1,"complex":0}
    ...     fc.add_field(mscop,dpf.Field(nentities=i+10))
    >>> for i in range(0,20): #imaginary fields
    ...     mscop = {"time":i+1,"complex":1}
    ...     fc.add_field(mscop,dpf.Field(nentities=i+10))

    """

    def __init__(self, fields_container=None, server=None):
        """Initialize the scoping with either an optional scoping message
        or by connecting to a stub.
        """
        if server is None:
            server = dpf.core._global_server()

        self._server = server
        self._stub = self._connect()

        self._component_index = None  # component index
        self._component_info = None  # for norm/max/min

        Collection.__init__(
            self, types.field, collection=fields_container, server=self._server
        )

    def get_fields_by_time_complex_ids(self, timeid=None, complexid=None):
        """Retrieve fields at a requested time ID or complex ID.

        Parameters
        ----------
        timeid : int, optional
            Time ID or frequency ID, which is the one-based index of the
            result set.
        complexid : int, optional
            Complex ID, where ``1`` is for imaginary and ``0`` is for real.

        Returns
        -------
        fields : list[Field]
            Fields corresponding to the request.

        Examples
        --------
        Extract the fifth time set of a transient analysis.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> len(model.metadata.time_freq_support.time_frequencies)
        35
        >>> disp = model.results.displacement()
        >>> disp.inputs.time_scoping.connect([1,5])
        >>> fields_container = disp.outputs.fields_container()
        >>> field_set_5 =fields_container.get_fields_by_time_complex_ids(5)

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        return super()._get_entries(label_space)

    def get_field_by_time_complex_ids(self, timeid=None, complexid=None):
        """Retrieve a field at a requested time ID or complex ID.

        An exception is raised if the number of fields matching the request is
        greater than one.

        Parameters
        ----------
        timeid : int, optional
            Time ID or frequency ID, which is the one-based index of the
            result set.
        complexid : int, optional
            Complex ID, where ``1`` is for imaginary and ``0`` is for real.

        Returns
        -------
        fields : Field
            Field corresponding to the request

        Examples
        --------
        Extract the fifth time set of a transient analysis.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> len(model.metadata.time_freq_support.time_frequencies)
        35
        >>> disp = model.results.displacement()
        >>> disp.inputs.time_scoping.connect([1,5])
        >>> fields_container = disp.outputs.fields_container()
        >>> field_set_5 =fields_container.get_fields_by_time_complex_ids(5)

        """
        label_space = self.__time_complex_label_space__(timeid, complexid)
        return super()._get_entry(label_space)

    def __time_complex_label_space__(self, timeid=None, complexid=None):
        label_space = {}
        if timeid is not None:
            label_space["time"] = timeid
        if complexid is not None:
            label_space["complex"] = complexid
        return label_space

    def get_fields(self, label_space):
        """Retrieve the fields at a requested index or label space.

        Parameters
        ----------
        label_space : dict[str,int]
            Scoping of the requested fields. For example,
            ``{"time": 1, "complex": 0}``.

        Returns
        -------
        fields : list[Field]
            Fields corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> fc= dpf.FieldsContainer()
        >>> fc.labels =['time','complex']
        >>> #real fields
        >>> for i in range(0,20):
        ...     mscop = {"time":i+1,"complex":0}
        ...     fc.add_field(mscop,dpf.Field(nentities=i+10))
        >>> #imaginary fields
        >>> for i in range(0,20):
        ...     mscop = {"time":i+1,"complex":1}
        ...     fc.add_field(mscop,dpf.Field(nentities=i+10))

        >>> fields = fc.get_fields({"time":2})
        >>> # imaginary and real fields of time 2
        >>> len(fields)
        2

        """

        return super()._get_entries(label_space)

    def get_field(self, label_space_or_index):
        """Retrieves the field at a requested index or label space.

        An exception is raised if the number of fields matching the request is
        greater than one.

        Parameters
        ----------
        label_space_or_index : dict[str,int], int
            Scoping of the requested fields. For example,
            ``{"time": 1, "complex": 0}`` or the index of the field.

        Returns
        -------
        field : Field
            Field corresponding to the request.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> fc = dpf.fields_container_factory.over_time_freq_fields_container(
        ...     [dpf.Field(nentities=10)]
        ... )
        >>> field = fc.get_field({"time":1})

        """
        return super()._get_entry(label_space_or_index)

    def get_field_by_time_id(self, timeid=None):
        """Retrieves the complex field at a requested time.

        Parameters
        ----------
        timeid: int, optional
            Time ID, which is the one-based index of the result set.

        Returns
        -------
        fields : Field
            Fields corresponding to the request.
        """
        if not self.has_label("time"):
            raise dpf_errors.DpfValueError(
                "The fields container is not based on time scoping."
            )

        if self.has_label("complex"):
            label_space = self.__time_complex_label_space__(timeid, 0)
        else:
            label_space = self.__time_complex_label_space__(timeid)

        return super()._get_entry(label_space)

    def get_imaginary_fields(self, timeid=None):
        """Retrieve the complex fields at a requested time.

        Parameters
        ----------
        timeid: int, optional
            Time ID, which is the one-based index of the result set.

        Returns
        -------
        fields : list[Field]
            Fields corresponding to the request.
        """
        if not self.has_label("complex") or not self.has_label("time"):
            raise dpf_errors.DpfValueError(
                "The fields container is not based on time and complex scoping."
            )

        label_space = self.__time_complex_label_space__(timeid, 1)

        return super()._get_entries(label_space)

    def get_imaginary_field(self, timeid=None):
        """Retrieve the complex field at a requested time.

        Parameters
        ----------
        timeid: int, optional
            Time ID, which is the one-based index of the result set.

        Returns
        -------
        fields : Field
            Field corresponding to the request.
        """
        if not self.has_label("complex") or not self.has_label("time"):
            raise dpf_errors.DpfValueError(
                "The fields container is not based on time and complex scoping."
            )

        label_space = self.__time_complex_label_space__(timeid, 1)

        return super()._get_entry(label_space)

    def __getitem__(self, key):
        """Retrieve the field at a requested index.

        Parameters
        ----------
        key : int
            Index.

        Returns
        -------
        field : Field
            Field corresponding to the request.
        """
        return super().__getitem__(key)

    def add_field(self, label_space, field):
        """Add or update a field at a requested label space.

        Parameters
        ----------
        label_space : dict[str,int]
            Label space of the requested field. For example,
            {"time":1, "complex":0}.
        field : Field
            DPF field to add or update.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> fc= dpf.FieldsContainer()
        >>> fc.labels =['time','complex']
        >>> for i in range(0,20): #real fields
        ...     mscop = {"time":i+1,"complex":0}
        ...     fc.add_field(mscop,dpf.Field(nentities=i+10))
        >>> for i in range(0,20): #imaginary fields
        ...     mscop = {"time":i+1,"complex":1}
        ...     fc.add_field(mscop,dpf.Field(nentities=i+10))

        """
        super()._add_entry(label_space, field)

    def add_field_by_time_id(self, field, timeid=1):
        """Add or update a field at a requested time ID.

         Parameters
         ----------
         field : Field
             DPF field to add or update.
        timeid: int, optional
             Time ID for the requested time set. The default is ``1``.
        """
        labels = self.labels
        if not self.has_label("time") and (
            len(self.labels) == 0
            or (len(self.labels) == 1 and self.has_label("complex"))
        ):
            self.add_label("time")
        if len(self.labels) == 1:
            super()._add_entry({"time": timeid}, field)
        elif self.has_label("time") and self.has_label("complex") and len(labels) == 2:
            super()._add_entry({"time": timeid, "complex": 0}, field)
        else:
            raise dpf_errors.DpfValueError(
                "The fields container is not only based on time scoping."
            )

    def add_imaginary_field(self, field, timeid=1):
        """Add or update an imaginary field at a requested time ID.

        Parameters
        ----------
        field : Field
            DPF field to add or update.
        timeid: int, optional
            Time ID for the requested time set. The default is ``1``.
        """
        if not self.has_label("time") and (
            len(self.labels) == 0
            or (len(self.labels) == 1 and self.has_label("complex"))
        ):
            self.add_label("time")
        if (
            not self.has_label("complex")
            and len(self.labels) == 1
            and self.has_label("time")
        ):
            self.add_label("complex")
        if (
            self.has_label("time")
            and self.has_label("complex")
            and len(self.labels) == 2
        ):
            super()._add_entry({"time": timeid, "complex": 1}, field)
        else:
            raise dpf_errors.DpfValueError(
                "The fields container is not only based on time scoping."
            )

    def select_component(self, index):
        """Select fields containing only the component index.

        Fields can be selected only by component index as multiple fields may
        contain a different number of components.

        Parameters
        ----------
        index : int
            Index of the component.

        Returns
        -------
        fields : FieldsContainer
            Fields container with one component selected in each field.

        Examples
        --------
        Select using a component index.

        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> disp = model.results.displacement()
        >>> disp.inputs.time_scoping.connect([1,5])
        >>> fields_container = disp.outputs.fields_container()
        >>> disp_x_fields = fields_container.select_component(0)
        >>> my_field = disp_x_fields[0]

        """
        comp_select = dpf.core.Operator("component_selector_fc")
        comp_select.connect(0, self)
        comp_select.connect(1, index)
        return comp_select.outputs.fields_container.get_data()

    @property
    def time_freq_support(self):
        """Time frequency support."""
        return self._get_time_freq_support()

    @time_freq_support.setter
    def time_freq_support(self, value):
        return super()._set_time_freq_support(value)

    def deep_copy(self, server=None):
        """Create a deep copy of the fields container's data (and its fields) on a given server.

        This method is useful for passing data from one server instance to another.

        Parameters
        ----------
        server : ansys.dpf.core.server, optional
            Server with the channel connected to the remote or local instance.
            The default is ``None``, in which case an attempt is made to use the
            global server.

        Returns
        -------
        fields_container_copy : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> from ansys.dpf.core import examples
        >>> transient = examples.download_transient_result()
        >>> model = dpf.Model(transient)
        >>> disp = model.results.displacement()
        >>> disp.inputs.time_scoping.connect([1,5])
        >>> fields_container = disp.outputs.fields_container()
        >>> other_server = dpf.start_local_server(as_global=False)
        >>> deep_copy = fields_container.deep_copy(server=other_server)

        """
        fc = FieldsContainer(server=server)
        fc.labels = self.labels
        for i, f in enumerate(self):
            fc.add_field(self.get_label_space(i), f.deep_copy(server))
        try:
            fc.time_freq_support = self.time_freq_support.deep_copy(server)
        except:
            pass
        return fc

    def get_time_scoping(self):
        """Retrieves the time scoping containing the time sets.

        Returns
        -------
        scoping: Scoping
            Scoping containing the time set IDs available in the fields container.
        """
        return self.get_label_scoping("time")

    def __add__(self, fields_b):
        """Add two fields or two fields containers.

        Returns
        -------
        add : operators.math.add_fc
        """
        from ansys.dpf.core import dpf_operator
        from ansys.dpf.core import operators

        if hasattr(operators, "math") and hasattr(operators.math, "add_fc"):
            op = operators.math.add_fc(self, fields_b, server=self._server)
        else:
            op = dpf_operator.Operator("add_fc", server=self._server)
            op.connect(0, self)
            op.connect(1, fields_b)
        return op

    def __sub__(self, fields_b):
        """Subtract two fields or two fields containers.

        Returns
        -------
        minus : operators.math.minus_fc
        """
        from ansys.dpf.core import dpf_operator
        from ansys.dpf.core import operators

        if hasattr(operators, "math") and hasattr(operators.math, "minus_fc"):
            op = operators.math.minus_fc(server=self._server)
        else:
            op = dpf_operator.Operator("minus_fc", server=self._server)
        op.connect(0, self)
        op.connect(1, fields_b)
        return op

    def __pow__(self, value):
        if value != 2:
            raise ValueError('DPF only the value is "2" suppported')
        from ansys.dpf.core import dpf_operator
        from ansys.dpf.core import operators

        if hasattr(operators, "math") and hasattr(operators.math, "sqr_fc"):
            op = operators.math.sqr_fc(server=self._server)
        else:
            op = dpf_operator.Operator("sqr_fc", server=self._server)
        op.connect(0, self)
        op.connect(1, value)
        return op

    def __mul__(self, value):
        """Multiply two fields or two fields containers.

        Returns
        -------
        mul : operators.math.generalized_inner_product_fc
        """
        from ansys.dpf.core import dpf_operator
        from ansys.dpf.core import operators

        if hasattr(operators, "math") and hasattr(
            operators.math, "generalized_inner_product_fc"
        ):
            op = operators.math.generalized_inner_product_fc(server=self._server)
        else:
            op = dpf_operator.Operator(
                "generalized_inner_product_fc", server=self._server
            )
        op.connect(0, self)
        op.connect(1, value)
        return op



"""Installation file for python dpf module
"""
import os
from io import open as io_open

from setuptools import setup

install_requires = ["psutil", "progressbar2", "numpy", "ansys.grpc.dpf>=0.2.3"]


# Get version from version info
filepath = os.path.dirname(__file__)
__version__ = None
version_file = os.path.join(filepath, "ansys", "dpf", "core", "_version.py")
with io_open(version_file, mode="r") as fd:
    exec(fd.read())  # execute file from raw string

readme_file = os.path.join(filepath, "README.md")

setup(
    name="ansys-dpf-core",
    packages=[
        "ansys.dpf.core",
        "ansys.dpf.core.examples",
        "ansys.dpf.core.operators",
        "ansys.dpf.core.operators.averaging",
        "ansys.dpf.core.operators.filter",
        "ansys.dpf.core.operators.geo",
        "ansys.dpf.core.operators.invariant",
        "ansys.dpf.core.operators.logic",
        "ansys.dpf.core.operators.mapping",
        "ansys.dpf.core.operators.math",
        "ansys.dpf.core.operators.mesh",
        "ansys.dpf.core.operators.metadata",
        "ansys.dpf.core.operators.min_max",
        "ansys.dpf.core.operators.result",
        "ansys.dpf.core.operators.scoping",
        "ansys.dpf.core.operators.serialization",
        "ansys.dpf.core.operators.utility",
    ],
    version=__version__,
    description="DPF Python gRPC client",
    # long_description=io_open(readme_file, encoding="utf-8").read(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    package_data={
        "ansys.dpf.core.examples": [
            "ASimpleBar.rst",
            "static.rst",
            "complex.rst",
            "model_with_ns.rst",
            "file_cyclic.rst",
            "msup_transient_plate1.rst",
            "rth/rth_electric.rth",
            "rth/rth_steady.rth",
            "rth/rth_transient.rth",
            "sub/cp56.sub",
            "msup/file.mode",
            "msup/file.rst",
            "msup/file.rfrq",
            "distributed/file0.rst",
            "distributed/file1.rst",
        ]
    },
    author='ANSYS',
    author_email='ramdane.lagha@ansys.com',
    maintainer_email="pyansys.maintainers@ansys.com",
    python_requires=">=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*",
    install_requires=install_requires,
    extras_require={
        "plotting": ["pyvista>=0.24.0", "matplotlib==3.2"],
        "reporting": ["scooby"],
    },
    url='https://github.com/pyansys/pydpf-core',
    license='MIT',
)



from ansys.dpf.core.mapping_types import map_types_to_python
from ansys.dpf.core.common import types
from ansys.grpc.dpf import operator_pb2
import re


class Output:
    """
    Parameters
    ----------
    spec :

    pin :

    operator :
    """

    def __init__(self, spec, pin, operator):
        self._spec = spec
        self._operator = operator
        self._pin = pin
        self._python_expected_types = []
        for cpp_type in self._spec.type_names:
            self._python_expected_types.append(map_types_to_python[cpp_type])

    def get_data(self):
        """Retrieves the output of the operator."""
        type_output = self._spec.type_names[0]
        if type_output == "abstract_meshed_region":
            type_output = "meshed_region"

        elif type_output == "fields_container":
            type_output = types.fields_container
        elif type_output == "scopings_container":
            type_output = types.scopings_container
        elif type_output == "meshes_container":
            type_output = types.meshes_container

        return self._operator.get_output(self._pin, type_output)

    def __call__(self):
        return self.get_data()

    def __str__(self):
        docstr = self._spec.name
        if self._spec.optional:
            docstr += " (optional)"
        docstr += ", expects types:" + "\n"
        for exp_types in self._python_expected_types:
            docstr += "   -" + exp_types + "\n"
        if self._spec.document:
            docstr += "help: " + self._spec.document + "\n"
        return docstr


class _Outputs:
    """
    Parameters
    ----------
    dict_outputs : dict
        Dictionary of outputs.
    operator :

    """

    def __init__(self, dict_outputs, operator):
        self._dict_outputs = dict_outputs
        self._operator = operator
        self._outputs = []

    def _get_given_output(self, input_type_name):
        corresponding_pins = []
        for asked_camel_types in input_type_name:
            for pin in self._dict_outputs:
                for cpp_type in self._dict_outputs[pin].type_names:
                    if map_types_to_python[cpp_type] == asked_camel_types:
                        corresponding_pins.append(pin)
                    elif asked_camel_types == "Any":
                        corresponding_pins.append(pin)
        return corresponding_pins

    def __str__(self):
        docstr = "Available outputs:\n"
        for output in self._outputs:
            tot_string = str(output._spec.name)
            input_string = tot_string.split("\n")
            input_string1 = input_string[0]
            line = ["   ", "- ", input_string1]
            docstr += "{:<5}{:<4}{:<20}\n".format(*line)
            for inputstr in input_string:
                if inputstr != input_string1:
                    line = ["   ", "  ", inputstr]
                    docstr += "{:<5}{:<4}{:<20}".format(*line)
                    docstr += "\n"
        return docstr


def _clearRepeatedMessage(message):
    try:
        while True:
            message.pop(len(message) - 1)
    except:
        pass


def _make_printable_type(type):
    """Remove characters not allowed in function names in Python."""
    chars_to_remove = ["<", ">", "-", "!", "?", "."]
    rx = "[" + re.escape("_".join(chars_to_remove)) + "]"
    type = re.sub(rx, "_", type)
    type = type.replace("abstract_", "")
    return type


def _modify_output_spec_with_one_type(output_spec, type):
    if isinstance(output_spec, operator_pb2.PinSpecification):
        spec = (
            operator_pb2.PinSpecification()
        )  # create a copy of the pin spec with only one type
        spec.CopyFrom(output_spec)
        _clearRepeatedMessage(spec.type_names)
        spec.type_names.extend([type])
    else:
        spec = output_spec
        spec.type_names = [type]

    return spec


class Outputs(_Outputs):
    """
    dict_outputs : dict
        Dictionary of outputs.
    operator :

    """

    def __init__(self, dict_outputs, operator):
        super().__init__(dict_outputs, operator)
        for pin in self._dict_outputs:
            if (
                len(self._dict_outputs[pin].type_names) == 1
                and self._dict_outputs[pin] is not None
            ):
                output_name = self._dict_outputs[pin].name
                output = Output(self._dict_outputs[pin], pin, self._operator)
                self._outputs.append(output)
                if hasattr(self, output_name):  # for autogenerated code
                    setattr(self, "_" + output_name, output)
                else:
                    setattr(self, output_name, output)
            # generate 1 output by type name
            elif (
                len(self._dict_outputs[pin].type_names) > 1
                and self._dict_outputs[pin] is not None
            ):
                for type in self._dict_outputs[pin].type_names:
                    output_name = (
                        self._dict_outputs[pin].name
                        + "_as_"
                        + _make_printable_type(type)
                    )
                    output = Output(
                        _modify_output_spec_with_one_type(
                            self._dict_outputs[pin], type
                        ),
                        pin,
                        self._operator,
                    )
                    self._outputs.append(output)
                    if hasattr(self, output_name):  # for autogenerated code
                        setattr(self, "_" + output_name, output)
                    else:
                        setattr(self, output_name, output)
